#! /usr/bin/python



__author__ = 'Predrag Pejovic'
__license__ = 'GPLv3+'
__date__ = '2017-11-26'



################################################################################



from pylab import *
from oscusb import *
import usbtmc
import time
from engineeringnotation import *



################################################################################
#
# generator
#
gen = usbtmc.Instrument(0x0957, 0x0407)
gen.write('output off')
gen.write('function sinusoid')
gen.write('voltage:offset 0')
gen.write('output:load 9.9e37')
#
def genamp(amplitude):
    gen.write('voltage:amplitude ' + str(amplitude))
#
def genfreq(frequency):    
    gen.write('frequency ' + str(frequency))
#
def waigen():
    gr = '0'
    while gr != '1':
        gr = gen.ask('*opc?')
#
################################################################################



################################################################################
#
o = Oscilloscope(shy = True)
#
o.write('recall:setup factory')
#
o.write('select:ch1 on')
o.write('ch1:probe 1')
o.write('ch1:coupling dc')
o.write('ch1:bandwidth off')
o.write('ch1:position 0')
#
o.write('select:ch2 on')
o.write('ch2:probe 1')
o.write('ch2:coupling dc')
o.write('ch2:bandwidth off')
o.write('ch2:position 0')
#
o.write('select:math off')
#
o.write('horizontal:position 0')
#
o.write('trigger:main:edge:source ext')
o.write('trigger:main:level 0.7')
#
o.waituntilready()
#
################################################################################



################################################################################

twait = 0.2

wholef = array([
2.0, 4.0, 10.0, 20.0, 40.0, 100.0, 200.0, 400.0, 1e3, 2e3, 4e3, 10e3, 
20e3, 40e3, 100e3, 200e3, 400e3, 1e6, 2e6, 4e6, 10e6, 20e6])

timescales = 0.1 / wholef

def measureH(f, Vm = 5):
    
    if f < 2.0:
        f = 2.0
    elif f > 20.e6:
        f = 20.e6
    
    findex = where(wholef <= f)[0][-1]
    ratio = wholef[findex] / f
    ns = int(round(2500 * ratio))
    wt = 2.0 * pi * arange(ns) / ns
    s = 2 * sin(wt)
    c = 2 * cos(wt)

    genfreq(f)
    genamp(2 * Vm)
    waigen()
    
    time.sleep(twait)
    
    o.write('horizontal:scale ' + str(timescales[findex]))
    o.waituntilready()

    o.autorange(1)
    o.autorange(2)
    
    o.stop()
    v1 = o.getsamples(1, start = 1, stop = ns)
    v2 = o.getsamples(2, start = 1, stop = ns)
    o.run()


    V1s = mean(v1 * s)
    V1c = mean(v1 * c)
    V1m = sqrt(V1c**2 + V1s**2)
    phi1 = - arctan2(V1s, V1c)

    V2s = mean(v2 * s)
    V2c = mean(v2 * c)
    V2m = sqrt(V2c**2 + V2s**2)
    phi2 = - arctan2(V2s, V2c)

    H = V1m / V2m
    phideg = degrees(phi1 - phi2)
    if phideg < -180:
        phideg += 360
    elif phideg > 180:
        phideg -= 360
    
    return (H, phideg)

################################################################################



print
fmin = float(input('fmin [Hz] = '))
fmax = float(input('fmax [Hz] = '))
n = int(input('npoints = '))
Vm = float(input('Vgm [V] = '))
ffname = raw_input('file first name = ')
print



t1 = time.time()

fs = linspace(fmin, fmax, n)

convn = str(int(ceil(log10(n + 1))))
convs = '{:' + convn + 'd}/{:' + convn + 'd}: '

H = empty(n)
phideg = empty(n)



gen.write('output on')

for i in range(n):
    hpack = measureH(fs[i], Vm)
    H[i] = hpack[0]
    phideg[i] = hpack[1]
    print (convs + 'f = {:s}Hz, H = {:s}, phi = {:s}deg').\
    format(i + 1, n, toengstring(fs[i], '6.2f'), toengstring(H[i], '6.2f'), 
    toengstring(phideg[i]))
    
gen.write('output off')


    
close('all')

figure(1, figsize = (8, 6))
#
subplot(2, 1, 1)
plot(fs, H, 'b')
plot(fs, H, 'b.')
ylabel('|H|')
xlim(fmin, fmax)
grid()
#
subplot(2, 1, 2)
plot(fs, phideg, 'b')
plot(fs, phideg, 'b.')
ylabel('phi [deg]')
xlabel('f [Hz]')
xlim(fmin, fmax)
#
phimax = max(phideg)
phimin = min(phideg)
if phimax <= 90:
    if phimin <= -90:
        yticks([-180, -135, -90, -45, 0, 45, 90])
    else:
        yticks([-90, -45, 0, 45, 90])
else:
    if phimin <= -90:
        yticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
    else:
        yticks([-90, -45, 0, 45, 90, 135, 180])
#
grid()
#
tstmp = timestamp()
savefig(ffname + '__tf__' + tstmp + '.pdf')



np.save(ffname + '__f__' + tstmp + '.npy', fs)
np.save(ffname + '__H__' + tstmp + '.npy', H)
np.save(ffname + '__phi__' + tstmp + '.npy', phideg)



t2 = time.time()
deltat = t2 - t1
m = int(deltat) / 60
s = deltat - 60 * m

print
print 'completed in {:d} minutes {:.2f} seconds'.format(m, s)
print

